from pydantic import BaseModel, Field, ConfigDict
from typing import Optional, List
from decimal import Decimal


class RenegotiationCampaign(BaseModel):
    model_config = ConfigDict(extra="allow")

    id: str = Field(...)
    company_id: str = Field(...)
    start_date: Optional[str] = Field(default=None)
    end_date: Optional[str] = Field(default=None)
    initial_day_after_renegotiate: Decimal = Field(...)
    max_number_of_parcel: Decimal = Field(...)
    payment_methods: List[str] = Field(...)
    created_at: Decimal = Field(...)
    has_discount: bool = Field(...)
    created_by: str = Field(...)
    status: str = Field(...)
    customer_ids: List[str] = Field(...)
    available_payment_status: List[str] = Field(...)
    days_after_start: Decimal = Field(...)
    discount: Decimal = Field(...)
    updated_at: Decimal = Field(...)
    min_days_overdue: Optional[Decimal] = Field(default=None)
    updated_by: str = Field(...)
    amount: Decimal = Field(...)
    has_fines_and_fees: bool = Field(...)
    is_active: bool = Field(...)
    min_number_of_installments: Decimal = Field(...)
    title: str = Field(...)



from datetime import datetime
from decimal import Decimal

from neofin_toobox.configs.enums import TableConfigEnum
from neofin_toobox.exceptions.common_exceptions import CommonException
from neofin_toobox.repositories.dynamodb_repository import DynamoDbRepository

from typing import List, Dict, Any
from boto3.dynamodb.conditions import Key
import logging

logger = logging.getLogger(__name__)


class CustomerRepository(DynamoDbRepository):
    GSI_COMPANY_DOCUMENT = 'company_id-document'
    GSI_DOCUMENT = 'document'

    def __init__(self):
        super().__init__()
        self.table_customer = self.resource.Table(TableConfigEnum.CUSTOMER)

    def get_by_customer_id_and_company_id(self, customer_id: str, company_id: str) -> Dict[str, Any]:
        try:
            response = self.table_customer.get_item(Key = {'id': customer_id, 'company_id': company_id})
            return response.get('Item', {})
        except Exception as e:
            logger.error(f"Error querying customer: customer_id={customer_id}, error={str(e)}")
            raise CommonException("Failed to retrieve customer")

    def get_by_document_and_company(self, company_id: str, document: str) -> List[Dict[str, Any]]:
        try:
            customer_items = self._query_all_customers(company_id, document)

            if not customer_items:
                logger.info(
                    f"No customers found for company_id={company_id}, document={document}"
                )
                return []

            customer_items.sort(key=lambda x: x.get('set_up_at', ''))
            return customer_items

        except Exception as e:
            logger.error(f"Error querying customers: company_id={company_id}, document={document}, error={str(e)}")
            raise CommonException("Failed to retrieve customers") from e

    def get_by_document(self, document: str) -> List[Dict[str, Any]]:
        try:
            logger.debug(f"Querying customers by document: {document}")

            customer_items = self._query_customers_by_document(document)

            if not customer_items:
                logger.info(f"No customers found for document={document}")
                return []

            self._normalize_setup_timestamps(customer_items)

            customer_items.sort(key=lambda x: x['set_up_at'])
            return customer_items

        except Exception as e:
            logger.error(f"Error querying customers by document: document={document}, error={str(e)}")
            raise CommonException(f"Exception occurred when getting customer by document={document}") from e

    def _query_all_customers(self, company_id: str, document: str) -> List[Dict[str, Any]]:
        customer_items = []
        last_evaluated_key = None

        while True:
            query_params = {
                'IndexName': self.GSI_COMPANY_DOCUMENT,
                'KeyConditionExpression': (
                        Key('company_id').eq(company_id) &
                        Key('document').eq(document)
                )
            }

            if last_evaluated_key:
                query_params['ExclusiveStartKey'] = last_evaluated_key

            response = self.table_customer.query(**query_params)

            if response.get('Count', 0) > 0:
                customer_items.extend(response['Items'])

            last_evaluated_key = response.get('LastEvaluatedKey')
            if not last_evaluated_key:
                break

        return customer_items

    def _query_customers_by_document(self, document: str) -> List[Dict[str, Any]]:
        customer_items = []
        last_evaluated_key = None

        while True:
            query_params = {
                'IndexName': self.GSI_DOCUMENT,
                'KeyConditionExpression': Key('document').eq(document)
            }

            if last_evaluated_key:
                query_params['ExclusiveStartKey'] = last_evaluated_key

            response = self.table_customer.query(**query_params)

            if response.get('Count', 0) > 0:
                customer_items.extend(response['Items'])

            last_evaluated_key = response.get('LastEvaluatedKey')
            if not last_evaluated_key:
                break

        return customer_items

    @staticmethod
    def _normalize_setup_timestamps(customer_items: List[Dict[str, Any]]) -> None:
        """Ensure all customers have a valid set_up_at timestamp.

        Args:
            customer_items: List of customer dictionaries to normalize
        """
        current_timestamp = int(datetime.now().timestamp())

        for customer in customer_items:
            if 'set_up_at' not in customer or customer['set_up_at'] is None:
                customer['set_up_at'] = current_timestamp
                logger.warning(f"Customer missing set_up_at, using current timestamp: {current_timestamp}")

    def get_by_company_id(self, company_id: str) -> List[Dict[str, Any]]:
        try:
            logger.debug(f"Scanning customers by company_id: {company_id}")

            customer_items = self._scan_customers_by_company_id(company_id)

            if not customer_items:
                logger.info(f"No customers found for company_id={company_id}")
                return []

            customer_items.sort(key=lambda x: Decimal(x.get('set_up_at', 99999999999)))

            return customer_items

        except Exception as e:
            logger.error(f"Error scanning customers by company_id: company_id={company_id}, error={str(e)}")
            raise CommonException(f"Exception occurred when getting customers by company_id: {company_id}") from e

    def _scan_customers_by_company_id(self, company_id: str) -> List[Dict[str, Any]]:
        customer_items = []
        last_evaluated_key = None

        while True:
            scan_params = {
                'FilterExpression': Key('company_id').eq(company_id)
            }

            if last_evaluated_key:
                scan_params['ExclusiveStartKey'] = last_evaluated_key

            response = self.table_customer.scan(**scan_params)

            if response.get('Count', 0) > 0:
                customer_items.extend(response['Items'])

            last_evaluated_key = response.get('LastEvaluatedKey')
            if not last_evaluated_key:
                break

        return customer_items


from jinja2 import Environment, FileSystemLoader
import boto3
import os
from botocore.exceptions import ClientError, NoCredentialsError
from typing import List, Dict, Any, Optional
import logging


logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class SESAdapter:
    """
    Classe para envio de emails usando Amazon SES com templates Jinja2.

    Attributes:
        region_name (str): Região AWS para o serviço SES
        template_folder (str): Caminho para a pasta de templates
        default_source_email (str): Email remetente padrão
    """

    def __init__(
            self,
            region_name: str = 'us-east-1',
            template_folder: Optional[str] = None,
            default_source_email: str = "noreply@neofin.com.br"
    ):
        """
        Inicializa a classe SES.

        Args:
            region_name: Região AWS (padrão: us-east-1)
            template_folder: Caminho para templates (padrão: ./templates)
            default_source_email: Email remetente padrão
        """
        self.region_name = region_name
        self.default_source_email = default_source_email

        # Configurar pasta de templates
        if template_folder is None:
            template_folder = os.path.join(os.path.dirname(__file__), 'templates')

        self.template_folder = template_folder

        if not os.path.exists(self.template_folder):
            raise FileNotFoundError(f"Template folder not found: {self.template_folder}")

        self._initialize_clients()

    def _initialize_clients(self) -> None:
        """Inicializa os clientes SES e Jinja2."""
        try:
            self.ses_client = boto3.client('ses', region_name=self.region_name)
            self.jinja_env = Environment(
                loader=FileSystemLoader(self.template_folder),
                autoescape=True  # Segurança contra XSS
            )
            logger.info(f"SES client initialized in region: {self.region_name}")
        except NoCredentialsError as e:
            logger.error("AWS credentials not found")
            raise
        except Exception as e:
            logger.error(f"Error initializing clients: {e}")
            raise

    def render_template(self, template_name: str, context: Dict[str, Any]) -> str:
        """
        Renderiza um template Jinja2 com o contexto fornecido.

        Args:
            template_name: Nome do arquivo de template
            context: Dicionário com variáveis para o template

        Returns:
            String com o HTML renderizado

        Raises:
            FileNotFoundError: Se o template não for encontrado
            Exception: Para outros erros de renderização
        """
        try:
            template = self.jinja_env.get_template(template_name)
            rendered_content = template.render(context)
            logger.info(f"Template '{template_name}' rendered successfully")
            return rendered_content
        except FileNotFoundError as e:
            logger.error(f"Template '{template_name}' not found: {e}")
            raise
        except Exception as e:
            logger.error(f"Error rendering template '{template_name}': {e}")
            raise

    def send_email(
            self,
            to_addresses: List[str],
            subject: str,
            template_name: str,
            context: Dict[str, Any],
            source_email: Optional[str] = None,
            cc_addresses: Optional[List[str]] = None,
            bcc_addresses: Optional[List[str]] = None,
            reply_to_addresses: Optional[List[str]] = None,
            include_text_version: bool = False
    ) -> Dict[str, Any]:
        """
        Envia email usando Amazon SES.

        Args:
            to_addresses: Lista de emails destinatários
            subject: Assunto do email
            template_name: Nome do template Jinja2
            context: Contexto para renderização do template
            source_email: Email remetente (opcional)
            cc_addresses: Lista de emails em cópia (opcional)
            bcc_addresses: Lista de emails em cópia oculta (opcional)
            reply_to_addresses: Lista de emails para resposta (opcional)
            include_text_version: Se deve incluir versão texto simples

        Returns:
            Resposta do SES com MessageId

        Raises:
            ValueError: Para parâmetros inválidos
            ClientError: Para erros do SES
        """
        # Validations
        if not to_addresses:
            raise ValueError("Recipient list cannot be empty")

        if not subject.strip():
            raise ValueError("Subject cannot be empty")

        # Use default email if not specified
        source = source_email or self.default_source_email

        try:
            # Render HTML template
            body_html = self.render_template(template_name, context)

            # Configure recipients
            destination = {'ToAddresses': to_addresses}
            if cc_addresses:
                destination['CcAddresses'] = cc_addresses
            if bcc_addresses:
                destination['BccAddresses'] = bcc_addresses

            # Configure message body
            message_body = {
                'Html': {
                    'Data': body_html,
                    'Charset': 'UTF-8'
                }
            }

            if include_text_version:
                try:
                    text_template_name = template_name.replace('.html', '.txt')
                    body_text = self.render_template(text_template_name, context)
                    message_body['Text'] = {
                        'Data': body_text,
                        'Charset': 'UTF-8'
                    }
                except FileNotFoundError:
                    logger.warning(f"Text template '{text_template_name}' not found")

            # Send email
            response = self.ses_client.send_email(
                Source=source,
                Destination=destination,
                Message={
                    'Subject': {
                        'Data': subject,
                        'Charset': 'UTF-8'
                    },
                    'Body': message_body
                },
                ReplyToAddresses=reply_to_addresses or []
            )

            message_id = response['MessageId']
            logger.info(f"Email sent successfully. MessageId: {message_id}")

            return response

        except ClientError as e:
            error_code = e.response['Error']['Code']
            logger.error(f"SES error ({error_code}): {e}")
            raise
        except Exception as e:
            logger.error(f"Unexpected error sending email: {e}")
            raise

    def send_bulk_emails(
            self,
            email_data: List[Dict[str, Any]],
            template_name: str,
            source_email: Optional[str] = None
    ) -> List[Dict[str, Any]]:
        """
        Sends multiple emails with different contexts.

        Args:
            email_data: List of dictionaries with email data
                       Format: [{'to': ['email@example.com'], 'subject': 'Subject', 'context': {...}}]
            template_name: Template name to be used
            source_email: Sender email (optional)

        Returns:
            List with results of each send
        """
        results = []

        for i, data in enumerate(email_data):
            try:
                result = self.send_email(
                    to_addresses=data['to'],
                    subject=data['subject'],
                    template_name=template_name,
                    context=data['context'],
                    source_email=source_email
                )
                results.append({
                    'index': i,
                    'status': 'success',
                    'message_id': result['MessageId'],
                    'to': data['to']
                })

            except Exception as e:
                logger.error(f"Error sending email {i}: {e}")
                results.append({
                    'index': i,
                    'status': 'error',
                    'error': str(e),
                    'to': data['to']
                })

        return results

    def verify_email_address(self, email: str) -> bool:
        """
        Verifies if an email address is verified in SES.

        Args:
            email: Email address to be verified

        Returns:
            True if verified, False otherwise
        """
        try:
            response = self.ses_client.get_identity_verification_attributes(
                Identities=[email]
            )

            verification_attrs = response['VerificationAttributes']
            if email in verification_attrs:
                status = verification_attrs[email]['VerificationStatus']
                return status == 'Success'

            return False

        except ClientError as e:
            logger.error(f"Error verifying email {email}: {e}")
            return False

    def get_send_statistics(self) -> Dict[str, Any]:
        """
        Gets SES send statistics.

        Returns:
            Dictionary with send statistics
        """
        try:
            response = self.ses_client.get_send_statistics()
            return response['SendDataPoints']
        except ClientError as e:
            logger.error(f"Error getting statistics: {e}")
            raise


import json
from decimal import Decimal

from boto3.dynamodb.types import TypeDeserializer

deserializer = TypeDeserializer()

def deserialize_item(item):
    return {k: deserializer.deserialize(v) for k, v in item.items()}


class DecimalEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Decimal):
            if len(str(obj)) == 10:
                return int(obj)
            return str(obj)
        return json.JSONEncoder.default(self, obj)


from datetime import datetime, timezone, timedelta
from functools import wraps

from neofin_toobox.repositories.audit_repository import AuditRepository


def save(**kwargs_dec):
    def save_decorator(f):
        @wraps(f)
        def wrapper_save(*args, **kwargs):
            print('------- {} - audit ------'.format(f.__name__))
            try:
                app = kwargs_dec.get('app')

                json_body = app.current_request.json_body or {}
                pk = json_body.get(kwargs_dec.get('entity_pk'))

                due_date = ''
                if not pk:
                    if 'first_due_date' in json_body:
                        due_date = int(json_body.get('first_due_date'))
                    elif 'due_date' in json_body:
                        due_date = int(json_body.get('due_date'))
                    if len(str(due_date)) > 10:
                        due_date = int(due_date / 1000)
                    customer_document = ''.join(d for d in json_body.get('customer_document', '') if d.isdigit())
                    user_id = app.current_request.context.get('authorizer', {}).get('claims', {}).get(
                        'cognito:username')
                    pk = '{}#{}#{}'.format(user_id, customer_document, str(due_date))

                now = datetime.now(tz=timezone(timedelta(hours=-3))).isoformat(timespec='seconds')

                to_save = {
                    'resource': '{}#{}'.format(kwargs_dec.get('entity'), pk),
                    'date': '{}#{}#{}'.format(now, f.__name__, app.current_request.method),
                    'username': app.current_request.context.get('authorizer', {}).get('claims', {}).get(
                        'cognito:username'),
                    'request_json': json_body,
                    'request_dict': app.current_request.to_dict()
                }

                audit_repository = AuditRepository()
                audit_repository.put_item(to_save)

            except Exception as e:
                print(('save_audit', 'exception', f.__name__, e))
                return f(*args, **kwargs)
            else:
                return f(*args, **kwargs)

        return wrapper_save

    return save_decorator


from typing import List, Dict, Any, Optional
from boto3.dynamodb.conditions import Key
import logging

from neofin_toobox.configs.enums import TableConfigEnum
from neofin_toobox.repositories.dynamodb_repository import DynamoDbRepository
from neofin_toobox.models.renegotiation_campaign import RenegotiationCampaign

logger = logging.getLogger(__name__)


class RenegotiationCampaignRepository(DynamoDbRepository):
    """Repository class for renegotiation campaign data operations in DynamoDB."""

    GSI_COMPANY_ID = 'company_id'
    GSI_END_DATE = 'end_date'

    def __init__(self):
        """Initialize the renegotiation campaign repository."""
        super().__init__()
        self.table_renegotiation_campaign = self.resource.Table(TableConfigEnum.RENEGOTIATION_CAMPAIGNS)

    def put_renegotiation_campaign(self, renegotiation_campaign: RenegotiationCampaign) -> None:
        """Save a renegotiation campaign to DynamoDB.

        Args:
            renegotiation_campaign: The RenegotiationCampaign object to save

        Raises:
            Exception: If DynamoDB put operation fails
        """
        try:
            logger.debug(f"Saving renegotiation campaign: {renegotiation_campaign.id}")

            self.table_renegotiation_campaign.put_item(
                Item=renegotiation_campaign.model_dump()
            )

            logger.info(f"Successfully saved renegotiation campaign: {renegotiation_campaign.id}")

        except Exception as e:
            logger.error(f"Error saving renegotiation campaign {renegotiation_campaign.id}: {str(e)}")
            raise Exception(
                f"Problems putting renegotiation campaign {renegotiation_campaign.id} in Database: {str(e)}")

    def list_renegotiation_campaigns_by_company(self, company_id: str) -> List[Dict[str, Any]]:
        """Retrieve all renegotiation campaigns by company ID.

        Args:
            company_id: The company identifier

        Returns:
            List of RenegotiationCampaign objects sorted by created_at

        Raises:
            Exception: If DynamoDB query fails
        """
        try:
            logger.debug(f"Querying renegotiation campaigns by company_id: {company_id}")

            campaign_items = self._query_campaigns_by_company(company_id)

            if not campaign_items:
                logger.info(f"No renegotiation campaigns found for company_id={company_id}")
                return []

            campaign_items.sort(key=lambda x: x['created_at'])

            logger.info(f"Company ID: {company_id} - Found {len(campaign_items)} renegotiation campaigns")

            return campaign_items

        except Exception as e:
            logger.error(f"Error listing renegotiation campaigns by company: company_id={company_id}, error={str(e)}")
            raise Exception(f"Problems listing renegotiation campaigns in Database: {str(e)}")

    def list_renegotiation_campaigns_by_end_date(self, end_date: str) -> List[Dict[str, Any]]:
        """Retrieve all renegotiation campaigns by end date.

        Args:
            end_date: The campaign end date

        Returns:
            List of RenegotiationCampaign objects

        Raises:
            Exception: If DynamoDB query fails
        """
        try:
            logger.debug(f"Querying renegotiation campaigns by end_date: {end_date}")

            campaign_items = self._query_campaigns_by_end_date(end_date)

            if not campaign_items:
                logger.info(f"No renegotiation campaigns found for end_date={end_date}")
                return []


            logger.info(f"Found {len(campaign_items)} renegotiation campaigns for end_date: {end_date}")

            return campaign_items

        except Exception as e:
            logger.error(f"Error listing renegotiation campaigns by end_date: end_date={end_date}, error={str(e)}")
            raise Exception(f"Problems listing renegotiation campaigns by end date {end_date} in Database: {str(e)}")

    def get_renegotiation_campaign_by_id(self, renegotiation_campaign_id: str) -> Optional[Dict[str, Any]]:
        """Retrieve a single renegotiation campaign by ID.

        Args:
            renegotiation_campaign_id: The campaign identifier

        Returns:
            RenegotiationCampaign object if found, None otherwise

        Raises:
            Exception: If DynamoDB get operation fails
        """
        try:
            logger.debug(f"Getting renegotiation campaign by id: {renegotiation_campaign_id}")

            response = self.table_renegotiation_campaign.get_item(
                Key={'id': renegotiation_campaign_id}
            )

            if 'Item' not in response:
                logger.info(f"No renegotiation campaign found for id={renegotiation_campaign_id}")
                return None

            campaign = response['Item']

            logger.info(f"Successfully retrieved renegotiation campaign: {renegotiation_campaign_id}")

            return campaign

        except Exception as e:
            logger.error(f"Error getting renegotiation campaign: id={renegotiation_campaign_id}, error={str(e)}")
            raise Exception(
                f"Problems getting renegotiation campaign {renegotiation_campaign_id} in Database: {str(e)}")

    def _query_campaigns_by_company(self, company_id: str) -> List[Dict[str, Any]]:
        """Query all campaigns by company_id with pagination handling.

        Args:
            company_id: The company identifier

        Returns:
            List of campaign items from DynamoDB
        """
        campaign_items = []
        last_evaluated_key = None

        while True:
            query_params = {
                'IndexName': self.GSI_COMPANY_ID,
                'KeyConditionExpression': Key('company_id').eq(company_id)
            }

            if last_evaluated_key:
                query_params['ExclusiveStartKey'] = last_evaluated_key

            response = self.table_renegotiation_campaign.query(**query_params)

            # Add items if any exist
            if response.get('Count', 0) > 0:
                campaign_items.extend(response['Items'])

            # Check if there are more pages
            last_evaluated_key = response.get('LastEvaluatedKey')
            if not last_evaluated_key:
                break

        return campaign_items

    def _query_campaigns_by_end_date(self, end_date: str) -> List[Dict[str, Any]]:
        """Query all campaigns by end_date with pagination handling.

        Args:
            end_date: The campaign end date

        Returns:
            List of campaign items from DynamoDB
        """
        campaign_items = []
        last_evaluated_key = None

        while True:
            query_params = {
                'IndexName': self.GSI_END_DATE,
                'KeyConditionExpression': Key('end_date').eq(end_date)
            }

            if last_evaluated_key:
                query_params['ExclusiveStartKey'] = last_evaluated_key

            response = self.table_renegotiation_campaign.query(**query_params)

            # Add items if any exist
            if response.get('Count', 0) > 0:
                campaign_items.extend(response['Items'])

            # Check if there are more pages
            last_evaluated_key = response.get('LastEvaluatedKey')
            if not last_evaluated_key:
                break

        return campaign_items


from neofin_toobox.exceptions.common_exceptions import CommonException


class RolesRepositoryException(CommonException):
    """Exceção personalizada para erros do UserRepository"""
    pass


class RolesNotFoundException(CommonException):
    """Exceção para quando usuário não é encontrado"""
    pass